library(tm)
library(RTextTools)
library(ROCR)
library(ggplot2)

### console --------------------------------------------------------------------
options(max.print = 100)

### load data ------------------------------------------------------------------
data.corpus <- readRDS("./saved/corpus.rds")
data <- data.frame(
  meta(data.corpus, "label", type="indexed"),
  meta(data.corpus, "file",  type="indexed")
)
data.dtm <- readRDS("./saved/dtmatrix.rds")

### training & testing datasets ------------------------------------------------
testSet.files <- c(
    "T26_2012_Costa_Rica_earthquake"
  , "T26_2012_Philipinnes_floods"
  , "T26_2012_Typhoon_Pablo"
  , "T26_2013_Spain_train_crash"
  , "T26_2012_Colorado_wildfires"
)
testSet  <- which(data$file %in% testSet.files)
trainSet <- setdiff(1:nrow(data), testSet)
cat(sprintf("train/test: %d / %d (%.1f%% / %.1f%%)",
            length(trainSet), length(testSet),
            length(trainSet)/nrow(data)*100,
            length(testSet) /nrow(data)*100
))

### SVM model ------------------------------------------------------------------
container <- create_container(
  matrix = data.dtm,
  labels = factor(data$label, levels = c(0,1), labels = c("off","on")),
  trainSize = trainSet, testSize = testSet,
  virgin = FALSE
)
# time: 21-43 min
model <- train_model(container, "SVM")
saveRDS(model, file = "./saved/model.svm.rds")

### predict on test set --------------------------------------------------------
model <- readRDS(file = "./saved/model.svm.rds")
# predict <- classify_model(container, model)
# predict.labels <- as.numeric(levels(predict$SVM_LABEL))[predict$SVM_LABEL]
predict <- predict(model, container@classification_matrix, probability=TRUE)

# which of lables is "on-topic"
label_on <- which(dimnames(attr(predict, "probabilities"))[[2]] == "on")
# cat(sprintf("\n%s - %d", model$levels, model$labels))
# cat(sprintf("%f / %f - %f\n",
#   attr(predict, "probabilities")[, 1],
#   attr(predict, "probabilities")[, 2],
#   data$label[testSet]
# ))

ggplot() + geom_density(alpha = 0.5, aes(
  fill = factor(data$label[testSet], labels = c("off-topic", "on-topic")),
  x = as.vector( attr(predict, "probabilities")[, label_on] ),
  y = ..count../max(..count..)
))

### accuracy metrics -----------------------------------------------------------
rocr.pred <- prediction(
  predictions = as.vector( attr(predict, "probabilities")[, label_on] ),
  labels      = data$label[testSet]
)
rocr.perf    <- performance(rocr.pred, measure = "acc", x.measure = "cutoff")
# plot(rocr.perf)
cutoff.index <- which.max(rocr.perf@y.values[[1]])
cutoff.value <- rocr.perf@x.values[[1]][cutoff.index]
cat(sprintf("cut-off: %.2f / prec: %.1f%% rec: %.1f%% auc: %.3f\n",
  cutoff.value,
  performance(rocr.pred, measure = "prec")@y.values[[1]][cutoff.index]*100,
  performance(rocr.pred, measure = "rec" )@y.values[[1]][cutoff.index]*100,
  performance(rocr.pred, measure = "auc" )@y.values[[1]]
))

### accuracy by file -----------------------------------------------------------
test <- data[testSet, ]
test[, "predict"] <- as.vector( attr(predict, "probabilities")[, label_on] )

testSet.quality <- do.call(rbind, lapply(testSet.files, function(file) {
  rocr.pred <- prediction(
    predictions = test[test$file == file, "predict"],
    labels      = test[test$file == file, "label"  ]
  )

  rocr.perf.prec <- performance(rocr.pred, "prec")
  rocr.perf.rec  <- performance(rocr.pred, "rec" )
  rocr.perf.auc  <- performance(rocr.pred, "auc" )

  cutoff.index <- which.min(abs(unlist(rocr.perf.rec@x.values) - cutoff.value))

  return(data.frame(
      "cut"  =  rocr.perf.rec@x.values[[1]][cutoff.index],
      "prec" = rocr.perf.prec@y.values[[1]][cutoff.index],
      "rec"  =  rocr.perf.rec@y.values[[1]][cutoff.index],
      "auc"  =  rocr.perf.auc@y.values[[1]],
      "file"  = file,
      "count" = nrow( test[test$file == file & test$label == 1, ] )
  ))
}))

# cat(sprintf("\n> %-30s (%3d): prec: %.1f%% rec: %.1f%% auc: %.3f",
cat(sprintf("\n> %-30s (%3d): prec: %.1f rec: %.1f auc: %.3f",
  testSet.quality$file,
  testSet.quality$count,
  testSet.quality$prec * 100,
  testSet.quality$rec  * 100,
  testSet.quality$auc
))

### test speed -----------------------------------------------------------------
# load(file = "./saved/model.svm.rda")
# system.time( predict(model, newdata = data.dtm, probability = TRUE) )
# 
# benchmark({
#   predict(model, newdata = data.dtm, probability = TRUE)
#   },
#   replications = 5
# )

# sLDA: 3303.832 / SVM: 665.416
# 3303.832 / 665.416

rm(
  container, cutoff.index, cutoff.value, data, data.corpus, data.dtm,
  label_on, model, predict, rocr.perf, rocr.pred, test, testSet, testSet.files,
  testSet.quality, trainSet
)

