library(lda)
library(tm)
library(topicmodels)
library(ROCR)
library(parallel)

### console --------------------------------------------------------------------
options(max.print = 50)

### load data ------------------------------------------------------------------
data.corpus <- readRDS("./saved/corpus.rds")
data <- data.frame(
  meta(data.corpus, "label", type="indexed"),
  meta(data.corpus, "file",  type="indexed")
)
data.dtm <- readRDS("./saved/dtmatrix.rds")
lda.corpus <- dtm2ldaformat(data.dtm, omit_empty=FALSE) # already cleaned

### training & testing datasets ------------------------------------------------
testSet.files <- c(
    "T26_2012_Costa_Rica_earthquake"
  , "T26_2012_Philipinnes_floods"
  , "T26_2012_Typhoon_Pablo"
  , "T26_2013_Spain_train_crash"
  , "T26_2012_Colorado_wildfires"
)
testSet  <- which(data$file %in% testSet.files)
trainSet <- setdiff(1:nrow(data), testSet)
cat(sprintf("train/test: %d / %d (%.1f%% / %.1f%%)",
            length(trainSet), length(testSet),
            length(trainSet)/(length(trainSet) + length(testSet))*100,
            length(testSet)/(length(trainSet) + length(testSet))*100
))

### sLDA model -----------------------------------------------------------------
topicsCnt <- 100

models.list <- mclapply(1:10, mc.cores = 3, FUN = function(x) {
  model <- slda.em(
    documents = lda.corpus$documents[trainSet],
    K = topicsCnt,
    vocab = lda.corpus$vocab,
    num.e.iterations = 10,
    num.m.iterations =  4,
    alpha = 1.0, eta = 0.1,
    annotations = data$label[trainSet],
    params = sample(c(0,1), topicsCnt, replace=TRUE),
    variance = 0.25,
    logistic = FALSE,    # default
    lambda = 1.0,
    regularise = FALSE,  # default
    method = "sLDA"
    )
  return(model)
})
save(models.list, file = "./saved/slda.models.list.rda")

## predict on test set
# load(file = "./saved/slda.models.list.rda")
predict.list <- mclapply(models.list, mc.cores = 3, FUN = function(m) {
  predict <- slda.predict(
    documents = lda.corpus$documents[testSet],
    topics = m$topics,
    model = m$model,
    alpha = 1.0,
    eta = 0.1,
    num.iterations = 1000,   # default
    average.iterations = 50, # default
    trace = 0L               # default 0L
  )
  return(predict)
})
rm(models.list)
save(predict.list, file = "./saved/slda.predict.list.rda")

### accuracy -------------------------------------------------------------------
# load(file = "./saved/slda.predict.list.rda")
rocr.pred <- prediction(
  predictions = predict.list,
  labels      = rep(list(data$label[testSet]), length(predict.list))
)

rocr.perf.acc  <- performance(rocr.pred, "acc")
rocr.perf.rec  <- performance(rocr.pred, "rec")
rocr.perf.prec <- performance(rocr.pred, "prec")
rocr.perf.auc  <- performance(rocr.pred, "auc")

max.acc <- lapply(rocr.perf.acc@y.values, which.max)
models.quality <- data.frame(
  "cut"  = sapply(1:10, function(i) { pos <- max.acc[[i]]; rocr.perf.acc@x.values[[i]][pos] }),
  "prec" = sapply(1:10, function(i) { pos <- max.acc[[i]]; rocr.perf.prec@y.values[[i]][pos]}),
  "rec"  = sapply(1:10, function(i) { pos <- max.acc[[i]]; rocr.perf.rec@y.values[[i]][pos] }),
  "auc"  = unlist(rocr.perf.auc@y.values)
)

cat(sprintf("\n[%2s] cut-off: %.2f / prec: %.1f%% rec: %.1f%% auc: %.3f",
  rownames(models.quality),
  models.quality$cut,
  models.quality$prec * 100,
  models.quality$rec  * 100,
  models.quality$auc
))

### accuracy by file -----------------------------------------------------------
test <- data[testSet, ]
test[, "predict"] <- predict.list[[which.max(models.quality$auc)]]

cutoff.value <- models.quality[which.max(models.quality$auc), "cut"]

testSet.quality <- do.call(rbind, lapply(testSet.files, function(file) {
  rocr.pred <- prediction(
    predictions = test[test$file == file, "predict"],
    labels      = test[test$file == file, "label"  ]
  )

  rocr.perf.rec  <- performance(rocr.pred, "rec")
  rocr.perf.prec <- performance(rocr.pred, "prec")
  rocr.perf.auc  <- performance(rocr.pred, "auc")

  cutoff.index <- which.min(abs(unlist(rocr.perf.rec@x.values) - cutoff.value))

  return(data.frame(
      "cut"  =  rocr.perf.rec@x.values[[1]][cutoff.index],
      "prec" = rocr.perf.prec@y.values[[1]][cutoff.index],
      "rec"  =  rocr.perf.rec@y.values[[1]][cutoff.index],
      "auc"  =  rocr.perf.auc@y.values[[1]],
      "file"  = file,
      "count" = nrow( test[test$file == file & test$label == 1, ] )
  ))
}))

cat(sprintf("\n> %-30s (%3d): prec: %.1f%% rec: %.1f%% auc: %.3f",
  testSet.quality$file,
  testSet.quality$count,
  testSet.quality$prec * 100,
  testSet.quality$rec  * 100,
  testSet.quality$auc
))

### test speed -----------------------------------------------------------------
# library(rbenchmark)
# load(file = "./saved/slda.models.list.rda")
# model <- model.list[[2]]
# rm(model.list)
# 
# benchmark({
#   slda.predict(
#     documents = lda.corpus$documents,
#     topics = model$topics,
#     model  = model$model,
#     alpha  = 1.0,
#     eta    = 0.1,
#     num.iterations = 1000,   # default
#     average.iterations = 50, # default
#     trace = 0L               # default 0L
#   )},
#   replications = 5
# )


rm(
  cutoff.value, data, data.corpus, data.dtm, lda.corpus, max.acc,
  models.quality, predict.list, rocr.perf.acc, rocr.perf.auc, rocr.perf.prec,
  rocr.perf.rec, rocr.pred, test, testSet, testSet.files, testSet.quality,
  topicsCnt, trainSet
)
