library(tm)
library(RTextTools)
library(ROCR)
library(ggplot2)
library(reshape2)
library(parallel)

### console --------------------------------------------------------------------
options(max.print = 100)

### load data ------------------------------------------------------------------
data.corpus <- readRDS("./saved/corpus.rds")
data <- data.frame(
  meta(data.corpus, "label", type="indexed"),
  meta(data.corpus, "file",  type="indexed")
)
data.dtm <- readRDS("./saved/dtmatrix.rds")

### training & testing datasets ------------------------------------------------
trainSet.files <- c(
    "T6_2013_Queensland_Floods"   # 8732
  , "T6_2013_Alberta_Floods"      # 9069
  , "T26_2013_Manila_floods"      # 1167
  , "T26_2013_Sardinia_floods"    #  180
  , "T26_2012_Philipinnes_floods" # 1318
)
testSet.files <- c(
    "T26_2013_Colorado_floods"    # 1664
)
trainSet <- which(data$file %in% trainSet.files)
testSet  <- which(data$file %in% testSet.files)

cat(sprintf("train/test: %d / %d (%.1f%% / %.1f%%)",
            length(trainSet), length(testSet),
            length(trainSet)/(length(trainSet)+length(testSet))*100,
            length(testSet) /(length(trainSet)+length(testSet))*100
))

### SVM ------------------------------------------------------------------------

model.size <- seq(1000, 20000, 1000)
model.size <- rep(model.size, 3)

model.quality <- mclapply(model.size, mc.cores = 3, FUN = function(iter.size) {
  # iter.size <- 1000
  ### train and test model -----------------------------------------------------
  container <- create_container(
    matrix = data.dtm,
    labels = factor(data$label, levels = c(0,1), labels = c("off","on")),
    trainSize = sort( sample(trainSet, size = iter.size, replace = FALSE) ),
    testSize = testSet,
    virgin = FALSE
  )
  model <- train_model(container, "SVM")
  predict <- predict(model, container@classification_matrix, probability=TRUE)
  label_on <- which(dimnames(attr(predict, "probabilities"))[[2]] == "on")

  ### accuracy metrics ---------------------------------------------------------
  rocr.pred <- prediction(
    predictions = as.vector( attr(predict, "probabilities")[, label_on] ),
    labels      = data$label[testSet]
  )
  rocr.perf    <- performance(rocr.pred, measure = "acc", x.measure = "cutoff")
  cutoff.index <- which.max(rocr.perf@y.values[[1]])

  return(c(
    "prec" = performance(rocr.pred, measure = "prec")@y.values[[1]][cutoff.index],
    "rec"  = performance(rocr.pred, measure = "rec" )@y.values[[1]][cutoff.index],
    "auc"  = performance(rocr.pred, measure = "auc" )@y.values[[1]]
  ))
})

model.quality <- do.call(rbind, model.quality)
model.quality <- as.data.frame.matrix(model.quality)
model.quality$size <- model.size

if (file.exists("./saved/training_size_svm.csv") == TRUE) {
  result <- read.csv("./saved/training_size_svm.csv", header = TRUE)
  result <- rbind(result, model.quality)
} else {
  result <- model.quality
}
write.csv(result, file = "./saved/training_size_svm.csv", row.names = FALSE)


### ----------------------------------------------------------------------------
rm(
  data, model.quality, result, data.corpus, data.dtm, model.size, testSet,
  testSet.files, trainSet, trainSet.files
)
